unit Unit1;

interface

uses SysUtils, forms, windows, inifiles, shellapi, ping;

   procedure startPinging;
   procedure logEvent(event: string);
   function ExecProg(FileName: String; Visibility: integer): longbool;


implementation



{
This procedure pings an i.p. address at particular intervals.
If it can't ping the address, it attempts to ping more frequently
but if there is still no successful ping after a certain period
it fires the alarm.
A successful ping resets the clock.
}
procedure startPinging;
var
   IniFile: TIniFile ;
   host, runprogram: string;
   lastGoodPing: DWORD;
   i, normalPingInterval, failurePingInterval, alertInterval, currentPingInterval: integer;
   lastPingInterval, beepLength: integer;
begin
   IniFile := TIniFile.Create(ExtractFilePath(Application.ExeName) + '\nopingalert.conf');
   try
      with IniFile do
      begin
         host := ReadString('main', 'host', 'localhost');
         normalPingInterval := ReadInteger('main', 'normalPingInterval', 30000);
         failurePingInterval := ReadInteger('main', 'failurePingInterval', 5000);
         alertInterval := ReadInteger('main', 'alertInterval', 90000);
         beepLength := ReadInteger('main', 'beepLength', 100);
         runprogram := ReadString('main', 'runProgram', '');
      end;
   finally
   	IniFile.Free ;
   end ;
   logEvent('Starting...');
   lastGoodPing := GetTickCount;

   while true do
   begin
      try
         if Ping.PingAddress(host) then
         begin
            logEvent('Ping ' + host + ' successful');
            lastGoodPing := GetTickCount;
            currentPingInterval := normalPingInterval;
         end
         else
         begin
            logEvent('Could not ping ' + host + ' :-(');
            currentPingInterval := failurePingInterval;  // increase frequency of ping attempts
         end;
         sleep(currentPingInterval);
         lastPingInterval := GetTickCount - lastGoodPing;
         logEvent('Last good ping was ' + IntToStr(lastPingInterval) + ' ms ago');
         if lastPingInterval > alertInterval then
         begin
            logEvent('No ping for ' + IntToStr(lastPingInterval) + 'ms');
            for i := 1 to beepLength do
            begin
               messagebeep($FFFFFFFF);
               sleep(10);
            end;
            if length(runprogram) > 0 then
            begin
               if ExecProg(runprogram, 0) then
                  logEvent('Launched ' +  runprogram)
               else
                  logEvent('Error running ' +  runprogram);
            end;
         end;
      except
         on e:exception do
         begin
            logEvent('Exception: ' + e.message);
         end;
      end;
   end;
   logEvent('Terminated'); // doubt this will ever occur, but just in case...
end;


procedure logEvent(event: string);
const
   logfile = 'nopingalert.log';
var
	f: textfile;
begin
   AssignFile(f, logfile);
   if FileExists(logfile) then
      Append(f)
   else
      Rewrite(f);
   writeln(f, DateTimeToStr(Now) + ' ' + event);
   CloseFile(f);
end;

function ExecProg(FileName: String; Visibility: integer): longbool;
var
  zAppName:array[0..512] of char;
  zCurDir:array[0..255] of char;
  WorkDir:String;
  StartupInfo:TStartupInfo;
  ProcessInfo:TProcessInformation;
begin
  StrPCopy(zAppName,FileName);
  GetDir(0,WorkDir);
  StrPCopy(zCurDir,WorkDir);
  FillChar(StartupInfo,Sizeof(StartupInfo),#0);
  StartupInfo.cb := Sizeof(StartupInfo);
  StartupInfo.dwFlags := STARTF_USESHOWWINDOW;
  StartupInfo.wShowWindow := Visibility;
  result := CreateProcess(nil,
    zAppName,                      { pointer to command line string }
    nil,                           { pointer to process security
attributes }

    nil,                           { pointer to thread security
attributes }
    false,                         { handle inheritance flag }
    CREATE_NEW_CONSOLE or          { creation flags }
    NORMAL_PRIORITY_CLASS,
    nil,                           { pointer to new environment block }
    nil,                           { pointer to current directory name }
    StartupInfo,                   { pointer to STARTUPINFO }
    ProcessInfo);
end;


end.
